proc NastranBase2PostCmdAssignMat { GDN w Menu geomlist } {
    $Menu del 0 end
    if { [catch { set whatuse [lindex [.gid.central.s info Project] 4] }] } {
	set whatuse GEOMETRYUSE
    }
    switch $whatuse {
	GEOMETRYUSE {
	    foreach i $geomlist {
		$Menu add command -label $i -command "DWAssignMaterials $GDN $w $i"
	    }
	}
	MESHUSE {
	    $Menu add command -label Elements -command "DWAssignMaterials $GDN $w"
	}
    }
}
proc NastranBasePostCmdAssignMat { GDN w Menu } {
    upvar \#0 $GDN GidData

    set MatName $GidData(MAT,matname)
    set _MatName [DWSpace2Under $MatName]
    set property [lindex [.central.s info Materials $_MatName] 4]
    set book [.central.s info Materials $_MatName BOOK]

    set LinesMats [list BAR BEAM TUBE VISCOUS_DAMPER SPRING DOF_SPRING]
    set SurfacesMats [list MEMBRANE PLATE PLANE_STRAIN SHEAR_PANEL BENDING_ONLY LAMINATE]
    set VolumesMats [list TETRAHEDRON HEXAHEDRON] 
    if { [lsearch $LinesMats $property] != -1 } {
	NastranBase2PostCmdAssignMat $GDN $w $Menu Lines
    } elseif { [lsearch $SurfacesMats $property] != -1 } {
	NastranBase2PostCmdAssignMat $GDN $w $Menu Surfaces
    } elseif { [lsearch $VolumesMats $property] != -1 } {
	NastranBase2PostCmdAssignMat $GDN $w $Menu Volumes
    } elseif { $book != "Material" && $book != "Tables"} {
	NastranPostCmdAssignMat $GDN $w $Menu
    } else {
	$Menu del 0 end
	$Menu add command -label "This material or table cannot be assigned"
    }
}

proc NastranBaseDWDrawMaterials { GDN w what } {
    upvar \#0 $GDN GidData

    set MatName $GidData(MAT,matname)
    set _MatName [DWSpace2Under $MatName]
    set book [.central.s info Materials $_MatName BOOK]
    if { $book != "Material" && $book != "Tables"} {
	NastranDWDrawMaterials $GDN $w $what
    } else {
	WarnWin "This material or table cannot be drawn"
    }
}

proc NastranBaseDWUnassignMat { w GDN } {
   upvar \#0 $GDN GidData

    set MatName $GidData(MAT,matname)
    set _MatName [DWSpace2Under $MatName]
    
    set book [.central.s info Materials $_MatName BOOK]
    if { $book != "Material" && $book != "Tables"} {
	NastranDWUnassignMat $w $GDN
    } else {
	WarnWin "This material or table cannot be unassigned"
    }
}




proc InitGIDProject { dir } {
    global MenuNames MenuEntries MenuCommands MenuAcceler
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP
    global ProgramName VersionNumber ProblemTypePriv NastranPriv
    global GidPriv
    
        
    set ProgramName "NASTRAN Interface" 
    set VersionNumber 1.4

    catch { unset ProblemTypePriv }

    set ProblemTypePriv(problemtypedir) $dir
    catch {
		GidShowBook intvdata "" 0
    }
    set ipos [lsearch $MenuNames Help]
    if { $ipos != -1 } {
	set MenuEntries($ipos) [linsert $MenuEntries($ipos) 0 "NASTRAN Interface tutorials" \
		"NASTRAN Interface help"]
	set MenuCommands($ipos) [linsert $MenuCommands($ipos) 0 \
		[list -np- HelpWindow "CUSTOM_HELP_FILE" \
		[file join $dir ../info nastran_tutorials.html]] \
		[list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
		[file join $dir ../info nastran_toc.html]]]
	set MenuAcceler($ipos) [linsert $MenuAcceler($ipos) 0 "" ""]

	lappend MenuEntries($ipos) "About NASTRAN Interface"
	lappend MenuCommands($ipos) [list -np- Splash $dir 0]
	lappend MenuAcceler($ipos) ""
    }
    set ipos [lsearch $MenuNamesP Help]
    if { $ipos != -1 } {
	set MenuEntriesP($ipos) [linsert $MenuEntriesP($ipos) 0 "NASTRAN Interface tutorials" \
		"NASTRAN Interface help"]
	set MenuCommandsP($ipos) [linsert $MenuCommandsP($ipos) 0 \
		[list -np- HelpWindow "CUSTOM_HELP_FILE" \
		[file join $dir ../info nastran_tutorials.html]] \
		[list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
		[file join $dir ../info nastran_toc.html]]]
	set MenuAccelerP($ipos) [linsert $MenuAccelerP($ipos) 0 "" ""]

	lappend MenuEntriesP($ipos) "About NASTRAN Interface"
	lappend MenuCommandsP($ipos) [list -np- Splash $dir 0]
	lappend MenuAccelerP($ipos) ""
    }
        
    set ipos [lsearch $MenuNamesP Files]
    set iposin [lsearch $MenuEntriesP($ipos) Import]
    set MenuEntriesP($ipos,$iposin) [linsert $MenuEntriesP($ipos,$iposin) 2 "FEMAP Info..."]
    set MenuCommandsP($ipos,$iposin) [linsert $MenuCommandsP($ipos,$iposin) 2 [list -np- FemapInfo ]]
    set MenuEntriesP($ipos,$iposin) [linsert $MenuEntriesP($ipos,$iposin) 3 "Import PUNCH"]
    set MenuCommandsP($ipos,$iposin) [linsert $MenuCommandsP($ipos,$iposin) 3 [list -np- PCH2GiD $dir 1 0]]
    set MenuEntriesP($ipos,$iposin) [linsert $MenuEntriesP($ipos,$iposin) 4 "Import PUNCH(Smooth)"]
    set MenuCommandsP($ipos,$iposin) [linsert $MenuCommandsP($ipos,$iposin) 4 [list -np- PCH2GiD $dir 1 1]]
    #set MenuAccelerP($ipos,$iposin) [linsert $MenuAccelerP($ipos,$iposin) 2 ""]
    
    set GidVersion [.gid.central.s info GiDVersion]
    set GidVersion [string trim $GidVersion]
    set GidVersion [string index $GidVersion 0]
    
    if { $GidVersion <= 6 } {
	set iposin2 [lsearch $MenuEntriesP($ipos,$iposin) "Image"]
	set MenuEntriesP($ipos,$iposin,$iposin2) $MenuEntriesP($ipos,$iposin,[expr $iposin2-2])
	catch {unset MenuEntriesP($ipos,$iposin,[expr $iposin2-2])}
	set MenuCommandsP($ipos,$iposin,$iposin2) $MenuCommandsP($ipos,$iposin,[expr $iposin2-2])
	catch {unset MenuCommandsP($ipos,$iposin,[expr $iposin2-2])}
    }
    set ipos [lsearch $MenuNames Files]
    if { $ipos != -1 } {
	set ipos2 [lsearch $MenuEntries($ipos) *Export*]
	if { $ipos2 != -1 } {
	    set ipos3 [lsearch $MenuEntries($ipos,$ipos2) "*Calculation file*"]
	    if { $ipos3 != -1 } {
		set MenuCommands($ipos,$ipos2) [lreplace $MenuCommands($ipos,$ipos2) \
			$ipos3 $ipos3 "-np- WriteCalcFileCheck"]
	    }
	}
    }
    
    set ipos [lsearch $MenuNames Calculate]
    if { $ipos != -1 } {
	set ipos2 [lsearch $MenuEntries($ipos) Calculate]
	if { $ipos2 != -1 } {
	    set MenuCommands($ipos) [lreplace $MenuCommands($ipos) \
			$ipos2 $ipos2 "-np- WriteCalcFileCheck"]
	}
    }
    foreach i [list PostCmdAssignMat DWDrawMaterials DWUnassignMat] {
	if { [info command Nastran$i] == "" } {
	    rename $i Nastran$i
	    rename NastranBase$i $i
	}
    }

    if { ![info exists  NastranPriv(infosplash)] } {
	catch { Splash $dir }
	set NastranPriv(infosplash) 1
    }
    catch { source [file join $dir  scripts HelpViewer.tcl] }
    catch { source [file join $dir scripts NastranPCH2GiD.tcl] }
    catch { source [file join $dir scripts Function.tcl] }
    catch { source [file join $dir scripts supergrid.tcl] }
    catch { source [file join $dir scripts VerifyModel.tcl] }
    catch { source [file join $dir scripts tkcheckdata.tcl] }
    catch { source [file join $dir scripts nassections.tcl] }
    catch { source [file join $dir scripts composite.tcl] }
    GidChangeDataLabel "Problem Data" ""
    GidChangeDataLabel "Conditions" ""
    GidChangeDataLabel "Materials" ""
    GidChangeDataLabel "Interval" ""
    
    GidAddUserDataOptionsMenu "Boundary Conditions" "NastranBoundaryconditionsSubmenu %W" 2
    GidAddUserDataOptionsMenu "Loads" "NastranLoadsSubmenu %W" 3
    GidAddUserDataOptions "Advanced Conditions" "GidOpenConditions Advanced_Conditions" 4
    GidAddUserDataOptions "Materials" "GidOpenMaterials Material" 5
    GidAddUserDataOptionsMenu "Properties" "NastranPropertiesSubmenu %W" 6
    GidAddUserDataOptionsMenu "Problem data" "NastranProblemdataSubmenu %W" 7
    GidAddUserDataOptions "---" "" 8   
        
    set GidPriv(ProgName) $ProgramName
    ChangeWindowTitle
    CreateTopMenus
}
proc NastranPropertiesSubmenu { w } {
    $w del 0 end
    $w add command -label "Property" -command [list GidOpenMaterials Property ]
    $w add command -label "Local Axes" -command [list GidOpenConditions Local_Axes ]
    $w configure -postcommand ""
}
proc NastranLoadsSubmenu { w } {
    $w del 0 end
    $w add command -label "Static Loads" -command [list GidOpenConditions Static_Loads ]
    $w add command -label "Dynamics Loads" -command [list GidOpenConditions Dynamics_Loads ]
    $w add command -label "Tables" -command [list GidOpenMaterials Tables ]
    $w configure -postcommand ""
}
proc NastranBoundaryconditionsSubmenu { w } {
    $w del 0 end
    $w add command -label "Constraints" -command [list GidOpenConditions Constraints ]
    $w add command -label "Connections" -command [list GidOpenConditions Connections ]
    $w configure -postcommand ""
}
proc NastranProblemdataSubmenu { w } {
    $w del 0 end
    $w add command -label "Executive Control" -command [list GidOpenProblemData Executive_Control]
    $w add command -label "Case Control" -command [list GidOpenProblemData Case_Control]
    $w add command -label "Dynamics" -command [list GidOpenProblemData Dynamics]
    $w configure -postcommand ""
}


proc EndGIDProject {} {
    global ProblemTypePriv
    
    if { [info exists ProblemTypePriv(condfuns)] } {
	foreach i $ProblemTypePriv(condfuns) {
	    rename $i {}
	    rename $i-base $i
	}
    }
    catch { destroy .gid.loadcases }
    foreach i [list PostCmdAssignMat DWDrawMaterials DWUnassignMat] {
	if { [info command Nastran$i] != "" } {
	    rename $i ""
	    rename Nastran$i $i
	}
    }
    global GidPriv
    set GidPriv(ProgName) GiD
    ChangeWindowTitle ""
}

proc Splash { dir { autodestroy 1 } } {
    global GIDDEFAULT VersionNumber ProgramName

    if { [.gid.central.s disable windows] } { return }

    if { [ winfo exist .splash]} {
	destroy .splash
	update
    }

    toplevel .splash
    wm withdraw .splash
        
    set im [image create photo -file [ file join $dir images \
	    [string tolower $ProgramName]-open.gif]]
    set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
    set y [expr [winfo screenheight .splash]/2-[image height $im]/2]

    wm geom .splash +$x+$y
    wm transient .splash .gid
    wm overrideredirect .splash 1
    pack [label .splash.l -image $im -relief ridge -bd 2]
    
    label .splash.lv -text $VersionNumber -foreground white -relief raised -bd 3 \
	    -background blue4 -font "verdana 8"
    place .splash.lv -x 62 -y 62
    bind .splash <1> "destroy .splash"
    bind .splash <KeyPress> "destroy .splash"
    grab .splash
    focus .splash
    update
    wm deiconify .splash
    raise .splash .gid
    update
    if { $autodestroy } {
	after 4000 "if { [ winfo exist .splash] } { 
	    destroy .splash
	}"
    }
    
}

proc PCH2GiD { dir reenterpost  smooth } {

    global DefaultSearchDirectory

    if { ![info exists DefaultSearchDirectory] || \
	     ![file isdirectory $DefaultSearchDirectory]} {
	set DefaultSearchDirectory [pwd]
    }
    set deft $DefaultSearchDirectory

    set filein [tk_getOpenFile -defaultextension ".pch"  -initialdir $deft \
		    -parent .gid -filetypes {{{PUNCH files} {.pch} } {{All files} {*} }}]
    if { $filein == "" } {
	return
    }
    set DefaultSearchDirectory [file dirname $filein]

    set aa [.central.s info Project]
    set ProjectName [lindex $aa 1]
    if { [file extension $ProjectName] == ".gid" } {
	set ProjectName [file root $ProjectName]
    }
    if { $ProjectName == "UNNAMED" } {
	tk_dialogRAM .gid.tmpwin error \
  		"Before importing, a project title is needed. Save project to get it" \
  		error 0 OK
  	return
    }

    set directory $ProjectName.gid
    if { [file pathtype $directory] == "relative" } {
	set directory [file join [pwd] $directory]
    }
    set fileout [file join $directory [file root [file tail $ProjectName]].flavia.res]
    
    PCH2GiD::TranslateW  $filein $fileout $smooth
    
    if { $reenterpost } {
	set DisableGraphics [.central.s disable graphics]
	set DisableWarnLine [.central.s disable warnline]
	.central.s disable graphics 1
	.central.s disable warnline 1
	.central.s process escape escape escape escape preprocess yes 
	.central.s process escape escape escape escape postprocess yes
	if { !$DisableGraphics } { .central.s disable graphics 0 }
	if { !$DisableWarnLine } { .central.s disable warnline 0 }
    }
}

proc FemapInfo { } {

    [ tk_dialogRAMFull .gid.alele {information window} \
	    "It is only possible to import FEMAP\nneutral files in ASCII format.\nTo\
	    obtain a FEMAP ASCII file in NE/NASTRAN go to NASTRAN Editor:\nMenu\
	    Setup, Default Analysis Options.\nSelect RSLTFILETYPE,FEMAP ASCII\
	    in Output Control Directives," \
	    "" "" gidquestionhead 0 OK ]
}